==> mythbuntu-common_0.64.tar.gz: mythbuntu-common-0.64/plugins/python/mythbuntu_repos.py <==
## -*- coding: utf-8 -*-
#
# «Mythbuntu Repos» - A Plugin for adding Mythbuntu Repos. (ie. -testing and
# Auto-Builds)
#
# Copyright (C) 2009, Thomas Mashos, for Mythbuntu
#
#
# Mythbuntu is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this application; if not, write to the Free Software Foundation, Inc., 51
# Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
################################################################################

from MythbuntuControlCentre.plugin import MCCPlugin
import os
import re
import commands
import urllib2
import shutil
import subprocess
import ConfigParser
from bzrlib.plugin import load_plugins
from bzrlib import branch, tree

class MythbuntuReposPlugin(MCCPlugin):
    """A Plugin for adding Mythbuntu Repos"""
    #
    #Load GUI & Calculate Changes
    #

    def __init__(self):
        #Initialize parent class
        information = {}
        information["name"] = "Repositories"
        information["icon"] = "emblem-downloads"
        information["ui"] = "tab_repos"
        MCCPlugin.__init__(self, information)
        self.CONFIGFILE = "/etc/default/mythbuntu-repos"
        self.USERHOME = os.path.expanduser("~")
        if not os.path.isdir(self.USERHOME+"/.mythbuntu"):
            os.mkdir(self.USERHOME+"/.mythbuntu")
        if not os.path.isfile(self.USERHOME+"/.mythbuntu/mythbuntu-repos.db"):
            try:
                self.downloadFile()
            except:
                shutil.copy("/usr/share/mythbuntu/plugins/mythbuntu-repos.db", self.USERHOME+"/.mythbuntu/")
        self.config = ConfigParser.ConfigParser()

    #Set mythtv versions
    def captureState(self):
        """Determines the state of the items on managed by this plugin
           and stores it into the plugin's own internal structures"""
        self.versions = []
        for line in open("/etc/lsb-release").xreadlines():
            if "DISTRIB_CODENAME" in line:
                line = line.strip("\n")
                throwaway, distro = line.split("=")
        NumRepos = 0
        for line in open(self.USERHOME+"/.mythbuntu/mythbuntu-repos.db").xreadlines():
            if distro in line:
                line = line.strip("\n")
                release, version = line.split("\t")
                self.versions.append(version)
                NumRepos += 1
            elif "MYTHTV_RELEASE" in line:
                line = line.strip("\n")
                VerName, self.CurVer = line.split("\t")
            elif "TRUNKPASS" in line:
                line = line.strip("\n")
                discard, self.TRUNKPASS = line.split("\t")
        self.changes = {}
        rei = 0
        while (rei <= NumRepos):
            self.builder.get_object('repobox').remove_text(0)
            rei = rei+1
        for item in self.versions:
            self.builder.get_object('repobox').append_text(item)
        if os.path.exists(self.CONFIGFILE):
            self.config.read(self.CONFIGFILE)
        try:
            self.changes['MythTVUpdatesActivated'] = self.config.getboolean("cfg", "ActivateMythTVUpdates")
        except:
            self.changes['MythTVUpdatesActivated'] = False
        try:
            self.changes['MythTVUpdatesRepo'] = self.config.get("cfg", "MythTVRepo")
        except:            
            self.changes['MythTVUpdatesRepo'] = self.versions[0]
        try:
            self.changes['ActivateMythbuntuUpdates'] = self.config.getboolean("cfg", "ActivateMythbuntuUpdates")
        except:
            self.changes['ActivateMythbuntuUpdates'] = False
        try:
            self.changes['ActivateXMLTVUpdates'] = self.config.getboolean("cfg", "ActivateXMLTVUpdates")
        except:
            self.changes['ActivateXMLTVUpdates'] = False

    def applyStateToGUI(self):
        """Takes the current state information and sets the GUI
           for this plugin"""
        self.mythtv_updates_checkbox.set_active(self.changes['MythTVUpdatesActivated'])
        self.repobox.set_sensitive(self.changes['MythTVUpdatesActivated'])
        try:
            self.repobox.set_active(self.versions.index(self.changes['MythTVUpdatesRepo']))
        except ValueError:
            self.repobox.set_active(0)
        self.mythbuntu_updates_checkbox.set_active(self.changes['ActivateMythbuntuUpdates'])
        self.xmltv_updates_checkbox.set_active(self.changes['ActivateXMLTVUpdates'])
        self.trunk_pass_ok.hide()

    def on_mythtv_updates_checkbox_toggled(self, widget, data=None):
        """Show the repobox if this is checked"""
        widget_was_visible = self.mythtv_updates_checkbox.get_active()
        if widget_was_visible:
            self.repobox.set_sensitive(True)
        else:
            self.repobox.set_sensitive(False)

    def on_dev_password_entry_changed(self, widget, data=None):
        """Require a password for using trunk"""
        if self.dev_password_entry.get_text() == self.TRUNKPASS:
            self.trunk_pass_error.hide()
            self.trunk_pass_ok.show()
        else:
            self.trunk_pass_error.show()
            self.trunk_pass_ok.hide()

    def on_repobox_changed(self, widget, data=None):
        """Check the version to know if it's trunk"""
        if not self.repobox.get_active_text() == None:
            SELVER = self.convertVersion(self.repobox.get_active_text())
            if SELVER > self.CurVer:
                self.trunk_block.show()
            else:
                self.trunk_block.hide()

    def convertVersion(self, VERSION):
        """Remove the trailing .x if it exists"""
        if VERSION.endswith(".x"):
            VERSION = VERSION.strip(".x")
        return VERSION

    def compareState(self):
        """Determines what items have been modified on this plugin"""
        MCCPlugin.clearParentState(self)
        SENDLIST = False
        SELVER = self.convertVersion(self.repobox.get_active_text())
        if self.mythtv_updates_checkbox.get_active() != self.changes['MythTVUpdatesActivated'] and self.mythtv_updates_checkbox.get_active() == False:
            self._markReconfigureRoot('MythTV-Updates-Activated', self.mythtv_updates_checkbox.get_active())
            SENDLIST = True
        if self.repobox.get_sensitive() == True:
            if self.repobox.get_active_text() != self.changes['MythTVUpdatesRepo'] or self.mythtv_updates_checkbox.get_active() != self.changes['MythTVUpdatesActivated']:
                if (SELVER > self.CurVer and self.dev_password_entry.get_text() == self.TRUNKPASS) or SELVER <= self.CurVer:
                    self._markReconfigureRoot('MythTV-Updates-Repo', self.repobox.get_active_text())
                    self._markReconfigureRoot('MythTV-Updates-Activated', self.mythtv_updates_checkbox.get_active())
                    SENDLIST = True
                elif self.mythtv_updates_checkbox.get_active() == False:
                    self._markReconfigureRoot('MythTV-Updates-Activated', self.mythtv_updates_checkbox.get_active())
        if self.mythbuntu_updates_checkbox.get_active() != self.changes['ActivateMythbuntuUpdates']:
            self._markReconfigureRoot('Mythbuntu-Updates-Activated', self.mythbuntu_updates_checkbox.get_active())
        if self.xmltv_updates_checkbox.get_active() != self.changes['ActivateXMLTVUpdates']:
            self._markReconfigureRoot('XMLTV-Updates-Activated', self.xmltv_updates_checkbox.get_active())
        if SENDLIST == True:
            self._markReconfigureRoot('Repo-list', self.versions)

    def refresh_button_clicked(self, widget, data=None):
        """Download a new db file if requested"""
        self.downloadFile()
        self.captureState()
        self.applyStateToGUI()

    def downloadFile(self):
        """Download files"""
        # Load the bzr plugins - the "launchpad" plugin provides support for the "lp:" shortcut
        load_plugins()

        b = branch.Branch.open('lp:~mythbuntu-dev/mythbuntu/mythbuntu-common')

        revision_number, revision_id = b.last_revision_info()

        repo = b.repository
        revision = repo.get_revision(revision_id)

        ## Get a revisiontree (needed to get the file)
        revtree = repo.revision_tree(revision_id)

        ## Download and save file
        b.lock_read()
        fileid = revtree.path2id('plugins/mythbuntu-repos.db')
        dbfile = open(self.USERHOME+"/.mythbuntu/mythbuntu-repos.db", "w")
        dbfile.write(revtree.get_file(fileid).read())
        dbfile.close()
        b.unlock()
        

    #
    # Process selected activities
    #

    def root_scripted_changes(self, reconfigure):
        """System-wide changes that need root access to be applied.
           This function is ran by the dbus backend"""
        if os.path.exists(self.CONFIGFILE):
            self.config.read("/etc/default/mythbuntu-repos")
        else:
            self.config.add_section("cfg")
        if "Repo-list" in reconfigure:
            for item in reconfigure["Repo-list"]:
                if item.endswith(".x"):
                    item = item.strip(".x")
                    subprocess.call(["apt-add-repository", "-r", "-y", "ppa:mythbuntu/"+item])
        if "MythTV-Updates-Activated" in reconfigure:
            if reconfigure["MythTV-Updates-Activated"]:
                repo = reconfigure["MythTV-Updates-Repo"]
                self.config.set("cfg", "ActivateMythTVUpdates", True)
                self.config.set("cfg", "MythTVRepo", repo)
                if repo.endswith(".x"):
                    repo = repo.strip(".x")
                subprocess.call(["apt-add-repository", "-y", "ppa:mythbuntu/"+repo])
            else:
                self.config.set("cfg", "ActivateMythTVUpdates", False)
        if "Mythbuntu-Updates-Activated" in reconfigure:
            if reconfigure["Mythbuntu-Updates-Activated"]:
                subprocess.call(["apt-add-repository", "-y", "ppa:mythbuntu/testing"])
                self.config.set("cfg", "ActivateMythbuntuUpdates", True)
            else:
                subprocess.call(["apt-add-repository", "-r", "-y", "ppa:mythbuntu/testing"])
                self.config.set("cfg", "ActivateMythbuntuUpdates", False)
        if "XMLTV-Updates-Activated" in reconfigure:
            if reconfigure["XMLTV-Updates-Activated"]:
                subprocess.call(["apt-add-repository", "-y", "ppa:mythbuntu/xmltv"])
                self.config.set("cfg", "ActivateXMLTVUpdates", True)
            else:
                subprocess.call(["apt-add-repository", "-r", "-y", "ppa:mythbuntu/xmltv"])
                self.config.set("cfg", "ActivateXMLTVUpdates", False)
        with open('/etc/default/mythbuntu-repos', 'wb') as configfile:
            self.config.write(configfile)

==> mythbuntu-common_0.67.tar.gz: mythbuntu-common-0.67/plugins/python/mythbuntu_repos.py <==
## -*- coding: utf-8 -*-
#
# «Mythbuntu Repos» - A Plugin for adding Mythbuntu Repos. (ie. -testing and
# Auto-Builds)
#
# Copyright (C) 2009, Thomas Mashos, for Mythbuntu
#
#
# Mythbuntu is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this application; if not, write to the Free Software Foundation, Inc., 51
# Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
################################################################################

from MythbuntuControlCentre.plugin import MCCPlugin
import os
import re
import commands
import urllib2
import shutil
import subprocess
import ConfigParser
from bzrlib.plugin import load_plugins
from bzrlib import branch, tree

class MythbuntuReposPlugin(MCCPlugin):
    """A Plugin for adding Mythbuntu Repos"""
    #
    #Load GUI & Calculate Changes
    #

    def __init__(self):
        #Initialize parent class
        information = {}
        information["name"] = "Repositories"
        information["icon"] = "emblem-downloads"
        information["ui"] = "tab_repos"
        MCCPlugin.__init__(self, information)
        self.CONFIGFILE = "/etc/default/mythbuntu-repos"
        self.USERHOME = os.path.expanduser("~")
        if not os.path.isdir(self.USERHOME+"/.mythbuntu"):
            os.mkdir(self.USERHOME+"/.mythbuntu")
        if not os.path.isfile(self.USERHOME+"/.mythbuntu/repos.db"):
            try:
                self.downloadFile()
            except:
                shutil.copy("/usr/share/mythbuntu/repos.db", self.USERHOME+"/.mythbuntu/")
        self.config = ConfigParser.ConfigParser()

    #Set mythtv versions
    def captureState(self):
        """Determines the state of the items on managed by this plugin
           and stores it into the plugin's own internal structures"""
        self.versions = []
        for line in open("/etc/lsb-release").xreadlines():
            if "DISTRIB_CODENAME" in line:
                line = line.strip("\n")
                throwaway, distro = line.split("=")
        NumRepos = 0
        dbfiles=[
        "/usr/share/mythbuntu/repos.db",
        self.USERHOME+"/.mythbuntu/repos.db"
        ]
        for f in dbfiles:
            for line in open(f).xreadlines():
                if distro in line:
                    line = line.strip("\n")
                    release, version = line.split("\t")
                    if not version in self.versions:
                        self.versions.append(version)
                    NumRepos += 1
                elif "MYTHTV_RELEASE" in line:
                    line = line.strip("\n")
                    VerName, self.CurVer = line.split("\t")
                elif "TRUNKPASS" in line:
                    line = line.strip("\n")
                    discard, self.TRUNKPASS = line.split("\t")
        self.changes = {}
        rei = 0
        while (rei <= NumRepos):
            self.builder.get_object('repobox').remove_text(0)
            rei = rei+1
        for item in self.versions:
            self.builder.get_object('repobox').append_text(item)
        if os.path.exists(self.CONFIGFILE):
            self.config.read(self.CONFIGFILE)
        try:
            self.changes['MythTVUpdatesActivated'] = self.config.getboolean("cfg", "ActivateMythTVUpdates")
        except:
            self.changes['MythTVUpdatesActivated'] = False
        try:
            self.changes['MythTVUpdatesRepo'] = self.config.get("cfg", "MythTVRepo")
        except:            
            self.changes['MythTVUpdatesRepo'] = self.versions[0]
        try:
            self.changes['ActivateMythbuntuUpdates'] = self.config.getboolean("cfg", "ActivateMythbuntuUpdates")
        except:
            self.changes['ActivateMythbuntuUpdates'] = False
        try:
            self.changes['ActivateXMLTVUpdates'] = self.config.getboolean("cfg", "ActivateXMLTVUpdates")
        except:
            self.changes['ActivateXMLTVUpdates'] = False

    def applyStateToGUI(self):
        """Takes the current state information and sets the GUI
           for this plugin"""
        self.mythtv_updates_checkbox.set_active(self.changes['MythTVUpdatesActivated'])
        self.repobox.set_sensitive(self.changes['MythTVUpdatesActivated'])
        try:
            self.repobox.set_active(self.versions.index(self.changes['MythTVUpdatesRepo']))
        except ValueError:
            self.repobox.set_active(0)
        self.mythbuntu_updates_checkbox.set_active(self.changes['ActivateMythbuntuUpdates'])
        self.xmltv_updates_checkbox.set_active(self.changes['ActivateXMLTVUpdates'])
        self.trunk_pass_ok.hide()

    def on_mythtv_updates_checkbox_toggled(self, widget, data=None):
        """Show the repobox if this is checked"""
        widget_was_visible = self.mythtv_updates_checkbox.get_active()
        if widget_was_visible:
            self.repobox.set_sensitive(True)
        else:
            self.repobox.set_sensitive(False)

    def on_dev_password_entry_changed(self, widget, data=None):
        """Require a password for using trunk"""
        if self.dev_password_entry.get_text() == self.TRUNKPASS:
            self.trunk_pass_error.hide()
            self.trunk_pass_ok.show()
        else:
            self.trunk_pass_error.show()
            self.trunk_pass_ok.hide()

    def on_repobox_changed(self, widget, data=None):
        """Check the version to know if it's trunk"""
        if not self.repobox.get_active_text() == None:
            SELVER = self.convertVersion(self.repobox.get_active_text())
            if SELVER > self.CurVer:
                self.trunk_block.show()
            else:
                self.trunk_block.hide()

    def convertVersion(self, VERSION):
        """Remove the trailing .x if it exists"""
        if VERSION.endswith(".x"):
            VERSION = VERSION.strip(".x")
        return VERSION

    def compareState(self):
        """Determines what items have been modified on this plugin"""
        MCCPlugin.clearParentState(self)
        SENDLIST = False
        SELVER = self.convertVersion(self.repobox.get_active_text())
        if self.mythtv_updates_checkbox.get_active() != self.changes['MythTVUpdatesActivated'] and self.mythtv_updates_checkbox.get_active() == False:
            self._markReconfigureRoot('MythTV-Updates-Activated', self.mythtv_updates_checkbox.get_active())
            SENDLIST = True
        if self.repobox.get_sensitive() == True:
            if self.repobox.get_active_text() != self.changes['MythTVUpdatesRepo'] or self.mythtv_updates_checkbox.get_active() != self.changes['MythTVUpdatesActivated']:
                if (SELVER > self.CurVer and self.dev_password_entry.get_text() == self.TRUNKPASS) or SELVER <= self.CurVer:
                    self._markReconfigureRoot('MythTV-Updates-Repo', self.repobox.get_active_text())
                    self._markReconfigureRoot('MythTV-Updates-Activated', self.mythtv_updates_checkbox.get_active())
                    SENDLIST = True
                elif self.mythtv_updates_checkbox.get_active() == False:
                    self._markReconfigureRoot('MythTV-Updates-Activated', self.mythtv_updates_checkbox.get_active())
        if self.mythbuntu_updates_checkbox.get_active() != self.changes['ActivateMythbuntuUpdates']:
            self._markReconfigureRoot('Mythbuntu-Updates-Activated', self.mythbuntu_updates_checkbox.get_active())
        if self.xmltv_updates_checkbox.get_active() != self.changes['ActivateXMLTVUpdates']:
            self._markReconfigureRoot('XMLTV-Updates-Activated', self.xmltv_updates_checkbox.get_active())
        if SENDLIST == True:
            self._markReconfigureRoot('Repo-list', self.versions)

    def refresh_button_clicked(self, widget, data=None):
        """Download a new db file if requested"""
        self.downloadFile()
        self.captureState()
        self.applyStateToGUI()

    def downloadFile(self):
        """Download files"""
        # Load the bzr plugins - the "launchpad" plugin provides support for the "lp:" shortcut
        load_plugins()

        b = branch.Branch.open('lp:~mythbuntu-dev/mythbuntu/mythbuntu-common')

        revision_number, revision_id = b.last_revision_info()

        repo = b.repository
        revision = repo.get_revision(revision_id)

        ## Get a revisiontree (needed to get the file)
        revtree = repo.revision_tree(revision_id)

        ## Download and save file
        b.lock_read()
        fileid = revtree.path2id('repos.db')
        dbfile = open(self.USERHOME+"/.mythbuntu/repos.db", "w")
        dbfile.write(revtree.get_file(fileid).read())
        dbfile.close()
        b.unlock()
        

    #
    # Process selected activities
    #

    def root_scripted_changes(self, reconfigure):
        """System-wide changes that need root access to be applied.
           This function is ran by the dbus backend"""
        if os.path.exists(self.CONFIGFILE):
            self.config.read("/etc/default/mythbuntu-repos")
        else:
            self.config.add_section("cfg")
        if "Repo-list" in reconfigure:
            for item in reconfigure["Repo-list"]:
                if item.endswith(".x"):
                    item = item.strip(".x")
                    subprocess.call(["apt-add-repository", "-r", "-y", "ppa:mythbuntu/"+item])
        if "MythTV-Updates-Activated" in reconfigure:
            if reconfigure["MythTV-Updates-Activated"]:
                repo = reconfigure["MythTV-Updates-Repo"]
                self.config.set("cfg", "ActivateMythTVUpdates", True)
                self.config.set("cfg", "MythTVRepo", repo)
                if repo.endswith(".x"):
                    repo = repo.strip(".x")
                subprocess.call(["apt-add-repository", "-y", "ppa:mythbuntu/"+repo])
            else:
                self.config.set("cfg", "ActivateMythTVUpdates", False)
        if "Mythbuntu-Updates-Activated" in reconfigure:
            if reconfigure["Mythbuntu-Updates-Activated"]:
                subprocess.call(["apt-add-repository", "-y", "ppa:mythbuntu/testing"])
                self.config.set("cfg", "ActivateMythbuntuUpdates", True)
            else:
                subprocess.call(["apt-add-repository", "-r", "-y", "ppa:mythbuntu/testing"])
                self.config.set("cfg", "ActivateMythbuntuUpdates", False)
        if "XMLTV-Updates-Activated" in reconfigure:
            if reconfigure["XMLTV-Updates-Activated"]:
                subprocess.call(["apt-add-repository", "-y", "ppa:mythbuntu/xmltv"])
                self.config.set("cfg", "ActivateXMLTVUpdates", True)
            else:
                subprocess.call(["apt-add-repository", "-r", "-y", "ppa:mythbuntu/xmltv"])
                self.config.set("cfg", "ActivateXMLTVUpdates", False)
        with open('/etc/default/mythbuntu-repos', 'wb') as configfile:
            self.config.write(configfile)
